class TriggerLogger(object):
    """
    A logger for the processing of a trigger.
    """

    def log_result(self, trigger, message, level=None):
        """
        Log the result of a trigger.
        :param level:
        """
        pass

    def log_message(self, trigger, message, level=None):
        """
        Log any message during the processing of a trigger.
        """
        pass
