# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2017-02-16 10:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('djtriggers', '0003_auto_20160512_0847'),
    ]

    operations = [
        migrations.AddField(
            model_name='trigger',
            name='number_of_tries',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='trigger',
            name='date_received',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
