# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-03 09:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djtriggers', '0005_trigger_successful'),
    ]

    operations = [
        migrations.AlterField(
            model_name='trigger',
            name='source',
            field=models.CharField(blank=True, db_index=True, max_length=150, null=True),
        ),
    ]
